@@ -26,6 +26,8 @@
 #include "syncdaemon-authentication.h"
 #include "syncdaemon-marshal.h"
 
+#define DEFAULT_ERROR_MESSAGE "No error specified"
+
 G_DEFINE_TYPE(SyncdaemonAuthentication, syncdaemon_authentication, G_TYPE_OBJECT)
 
 struct _SyncdaemonAuthenticationPrivate {
@@ -131,16 +133,21 @@ authorization_denied_cb (DBusGProxy *proxy, const gchar *app_name, gpointer user
 static void
 credentials_error_cb (DBusGProxy *proxy,
 		      const gchar *app_name,
-		      const gchar *error_message,
-		      const gchar *detailed_error,
+		      GHashTable *error_dict,
 		      gpointer user_data)
 {
+	gchar *error_message;
+
 	g_debug ("Authorization error");
 	if (g_strcmp0 (app_name, SSO_APP_NAME) != 0) {
 		g_debug ("Credentials are not for '" SSO_APP_NAME "', ignoring");
 		return;
 	}
 
+	error_message = g_hash_table_lookup (error_dict, "error_message");
+	if (error_message == NULL)
+		error_message = DEFAULT_ERROR_MESSAGE;
+
 	g_signal_emit (user_data, auth_signals[ERROR_SIGNAL], 0, error_message);
 }
 
@@ -159,7 +166,7 @@ syncdaemon_authentication_init (SyncdaemonAuthentication *auth)
 		g_error_free (error);
 	} else {
 		auth->priv->proxy = dbus_g_proxy_new_for_name (auth->priv->bus, "com.ubuntu.sso",
-							       "/credentials", "com.ubuntu.sso.ApplicationCredentials");
+							       "/com/ubuntu/sso/credentials", "com.ubuntu.sso.CredentialsManagement");
 		if (auth->priv->proxy != NULL) {
 			dbus_g_object_register_marshaller (_syncdaemon_marshal_VOID__STRING_POINTER,
 							   G_TYPE_NONE,
@@ -186,8 +193,9 @@ syncdaemon_authentication_init (SyncdaemonAuthentication *auth)
 
 			dbus_g_proxy_add_signal (auth->priv->proxy, "CredentialsError",
 						 G_TYPE_STRING,
-						 G_TYPE_STRING,
-						 G_TYPE_STRING,
+						 dbus_g_type_get_map ("GHashTable",
+								      G_TYPE_STRING,
+								      G_TYPE_STRING),
 						 G_TYPE_INVALID);
 			dbus_g_proxy_connect_signal (auth->priv->proxy, "CredentialsError",
 						     G_CALLBACK (credentials_error_cb), auth, NULL);
