@@ -11,4 +11,4 @@
 #
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
-""" Tests module """
+"""Tests module."""
