@@ -25,16 +25,14 @@ from xdg.BaseDirectory import (
     xdg_cache_home,
 )
 
-from ubuntuone.syncdaemon import (
-    config,
-)
+from ubuntuone.syncdaemon import config
 from contrib.testing.testcase import (
     BaseTwistedTestCase,
-    DBusTwistedTestCase,
     environ,
 )
 
 
+
 class TestConfigBasic(BaseTwistedTestCase):
     """Basic _Config object tests"""
 
@@ -238,7 +236,8 @@ class TestConfigBasic(BaseTwistedTestCase):
 
     def test_load_udf_autosubscribe(self):
         """Test load/set/override of udf_autosubscribe config value."""
-        conf_file = os.path.join(self.test_root, 'test_udf_autosubscribe_config.conf')
+        conf_file = os.path.join(self.test_root,
+                                 'test_udf_autosubscribe_config.conf')
         # write some throttling values to the config file
         with open(conf_file, 'w') as fp:
             fp.write('[__main__]\n')
@@ -278,11 +277,156 @@ class TestConfigBasic(BaseTwistedTestCase):
         overridden_opts = [('__main__', 'udf_autosubscribe', False)]
         conf.override_options(overridden_opts)
         self.assertFalse(conf.get_udf_autosubscribe())
-        self.assertFalse(conf.get_udf_autosubscribe() == conf_orig.get_udf_autosubscribe())
+        self.assertNotEqual(conf.get_udf_autosubscribe(),
+                            conf_orig.get_udf_autosubscribe())
         conf.save()
         conf_1 = config._Config(conf_file)
         self.assertEquals(True, conf_1.get_udf_autosubscribe())
 
+    def test_load_share_autosubscribe(self):
+        """Test load/set/override of share_autosubscribe config value."""
+        conf_file = os.path.join(self.test_root,
+                                 'test_share_autosubscribe_config.conf')
+        # write some throttling values to the config file
+        with open(conf_file, 'w') as fp:
+            fp.write('[__main__]\n')
+            fp.write('share_autosubscribe = True\n')
+
+        # keep a original around
+        conf_orig = config._Config(conf_file)
+
+        # load the config
+        conf = config._Config(conf_file)
+        self.assertTrue(conf.get_share_autosubscribe())
+        # change it to False
+        conf.set_share_autosubscribe(False)
+        self.assertFalse(conf.get_share_autosubscribe())
+        # save, load and check
+        conf.save()
+        conf_1 = config._Config(conf_file)
+        self.assertFalse(conf_1.get_share_autosubscribe())
+        # change it to True
+        conf.set_share_autosubscribe(True)
+        self.assertTrue(conf.get_share_autosubscribe())
+        # save, load and check
+        conf.save()
+        conf_1 = config._Config(conf_file)
+        self.assertTrue(conf_1.get_share_autosubscribe())
+
+        # load the config, check the override of the value
+        conf = config._Config(conf_file)
+        self.assertTrue(conf.get_share_autosubscribe())
+        overridden_opts = [('__main__', 'share_autosubscribe', False)]
+        conf.override_options(overridden_opts)
+        self.assertFalse(conf.get_share_autosubscribe())
+        self.assertNotEqual(conf.get_share_autosubscribe(),
+                            conf_orig.get_share_autosubscribe())
+        conf.save()
+        conf_1 = config._Config(conf_file)
+        self.assertEquals(True, conf_1.get_share_autosubscribe())
+
+    def test_load_autoconnect(self):
+        """Test load/set/override of autoconnect config value."""
+        conf_file = os.path.join(self.test_root,
+                                 'test_autoconnect_config.conf')
+        # ensure that autoconnect is True
+        with open(conf_file, 'w') as fp:
+            fp.write('[__main__]\n')
+            fp.write('autoconnect = True\n')
+
+        # keep a original around
+        conf_orig = config._Config(conf_file)
+
+        # assert default is correct
+        self.assertTrue(conf_orig.get_autoconnect(),
+                        'autoconnect is True by default.')
+
+        # load the config
+        conf = config._Config(conf_file)
+        self.assertTrue(conf.get_autoconnect())
+
+        # change it to False
+        conf.set_autoconnect(False)
+        self.assertFalse(conf.get_autoconnect())
+
+        # save, load and check
+        conf.save()
+        conf_1 = config._Config(conf_file)
+        self.assertFalse(conf_1.get_autoconnect())
+        # change it to True
+        conf.set_autoconnect(True)
+        self.assertTrue(conf.get_autoconnect())
+        # save, load and check
+        conf.save()
+        conf_1 = config._Config(conf_file)
+        self.assertTrue(conf_1.get_autoconnect())
+
+        # load the config, check the override of the value
+        conf = config._Config(conf_file)
+        self.assertTrue(conf.get_autoconnect())
+        overridden_opts = [('__main__', 'autoconnect', False)]
+        conf.override_options(overridden_opts)
+        self.assertFalse(conf.get_autoconnect())
+        self.assertNotEqual(conf.get_autoconnect(),
+                            conf_orig.get_autoconnect())
+        conf.save()
+        conf_1 = config._Config(conf_file)
+        self.assertEquals(True, conf_1.get_autoconnect())
+
+    def test_load_show_all_notifications(self):
+        """Test load/set/override of show_all_notifications config value."""
+        conf_file = os.path.join(self.test_root,
+                                 'test_show_all_notifications_config.conf')
+        # ensure that show_all_notifications is True
+        with open(conf_file, 'w') as fp:
+            fp.write('[notifications]\n')
+            fp.write('show_all_notifications = True\n')
+
+        # keep a original around
+        conf_orig = config._Config(conf_file)
+
+        # assert default is correct
+        self.assertTrue(conf_orig.get_show_all_notifications(),
+                        'show_all_notifications is True by default.')
+
+        # load the config
+        conf = config._Config(conf_file)
+        self.assertTrue(conf.get_show_all_notifications())
+
+        # change it to False
+        conf.set_show_all_notifications(False)
+        self.assertFalse(conf.get_show_all_notifications())
+
+        # save, load and check
+        conf.save()
+        conf_1 = config._Config(conf_file)
+        self.assertFalse(conf_1.get_show_all_notifications())
+        # change it to True
+        conf.set_show_all_notifications(True)
+        self.assertTrue(conf.get_show_all_notifications())
+        # save, load and check
+        conf.save()
+        conf_1 = config._Config(conf_file)
+        self.assertTrue(conf_1.get_show_all_notifications())
+
+        # load the config, check the override of the value
+        conf = config._Config(conf_file)
+        self.assertTrue(conf.get_show_all_notifications())
+        overridden_opts = [('notifications', 'show_all_notifications', False)]
+        conf.override_options(overridden_opts)
+        self.assertFalse(conf.get_show_all_notifications())
+        self.assertNotEqual(conf.get_show_all_notifications(),
+                            conf_orig.get_show_all_notifications())
+        conf.save()
+        conf_1 = config._Config(conf_file)
+        self.assertEquals(True, conf_1.get_show_all_notifications())
+
+    def test_load_branch_configuration(self):
+        """Check that the configuration from the branch is loaded."""
+        config_files = config.get_config_files()
+        rootdir = os.environ['ROOTDIR']
+        branch_config = os.path.join(rootdir, "data", config.CONFIG_FILE)
+        self.assertIn(branch_config, config_files)
 
 class ConfigglueParsersTests(BaseTwistedTestCase):
     """Tests for our custom configglue parsers"""
@@ -307,9 +451,9 @@ class ConfigglueParsersTests(BaseTwistedTestCase):
         bad_value = 'hola'
         invalid_value = None
         parser = config.log_level_parser
-        self.assertEquals(logging.INFO, parser(good_value))
-        self.assertEquals(logging.DEBUG, parser(bad_value))
-        self.assertRaises(TypeError, parser, invalid_value)
+        self.assertEqual(logging.INFO, parser(good_value))
+        self.assertEqual(logging.DEBUG, parser(bad_value))
+        self.assertEqual(logging.DEBUG, parser(invalid_value))
 
     def test_home_dir_parser(self):
         """Test home_dir_parser"""
@@ -345,72 +489,6 @@ class ConfigglueParsersTests(BaseTwistedTestCase):
         self.assertRaises(AttributeError, parser, invalid_value)
 
 
-class TestDBusConfig(DBusTwistedTestCase):
-
-    def test_get_throttling_limits(self):
-        """Test getting throttling limits"""
-        dbus_config = self.main.dbus_iface.config
-        limits = dbus_config.get_throttling_limits()
-        self.assertEquals(limits['download'], -1)
-        self.assertEquals(limits['upload'], -1)
-
-    def test_set_throttling_limits(self):
-        """Test setting throttling limits"""
-        dbus_config = self.main.dbus_iface.config
-        dbus_config.set_throttling_limits(10, 20)
-        limits = dbus_config.get_throttling_limits()
-        self.assertEquals(limits['download'], 10)
-        self.assertEquals(limits['upload'], 20)
-        conf = ConfigParser()
-        conf.read(config.get_user_config().config_file)
-        self.assertEquals(conf.get(config.THROTTLING, 'read_limit'), '10')
-        self.assertEquals(conf.get(config.THROTTLING, 'write_limit'), '20')
-
-    def test_enable_bandwidth_throttling(self):
-        """Test for enabling bandwidth throttling"""
-        dbus_config = self.main.dbus_iface.config
-        enabled = dbus_config.bandwidth_throttling_enabled()
-        self.assertFalse(enabled)
-        dbus_config.enable_bandwidth_throttling()
-        enabled = dbus_config.bandwidth_throttling_enabled()
-        self.assertTrue(enabled)
-        conf = ConfigParser()
-        conf.read(config.get_user_config().config_file)
-        self.assertEquals(conf.get(config.THROTTLING, 'on'), 'True')
-
-    def test_disable_bandwidth_throttling(self):
-        """Test for disabling bandwidth throttling"""
-        dbus_config = self.main.dbus_iface.config
-        enabled = dbus_config.bandwidth_throttling_enabled()
-        self.assertFalse(enabled)
-        # enable throttling
-        dbus_config.enable_bandwidth_throttling()
-        enabled = dbus_config.bandwidth_throttling_enabled()
-        self.assertTrue(enabled)
-        # now disable throttling and check
-        dbus_config.disable_bandwidth_throttling()
-        enabled = dbus_config.bandwidth_throttling_enabled()
-        self.assertFalse(enabled)
-        conf = ConfigParser()
-        conf.read(config.get_user_config().config_file)
-        self.assertEquals(conf.get(config.THROTTLING, 'on'), 'False')
-
-    def test_files_sync_enabled(self):
-        """Test for toggling files sync."""
-        dbus_config = self.main.dbus_iface.config
-        enabled = dbus_config.files_sync_enabled()
-        self.assertTrue(enabled)
-        dbus_config.set_files_sync_enabled(False)
-        enabled = dbus_config.files_sync_enabled()
-        self.assertFalse(enabled)
-        dbus_config.set_files_sync_enabled(True)
-        enabled = dbus_config.files_sync_enabled()
-        self.assertTrue(enabled)
-        conf = ConfigParser()
-        conf.read(config.get_user_config().config_file)
-        self.assertEquals(conf.get(config.MAIN, 'files_sync_enabled'), 'True')
-
-
 class SyncDaemonConfigParserTests(BaseTwistedTestCase):
     """Tests for SyncDaemonConfigParser"""
 
@@ -450,18 +528,19 @@ class SyncDaemonConfigParserTests(BaseTwistedTestCase):
         self.assertEquals(self.cp.get('logging', 'level').value, 10)
 
     def test_log_level_old_and_new_config(self):
-        """Test log_level upgrade hook with a mixed config"""
-        conf_file = os.path.join(self.test_root, 'test_old_and_new_config.conf')
-        # write some throttling values to the config file
+        """Test log_level upgrade hook with a mixed config."""
+        conf_file = os.path.join(self.test_root,
+                                 'test_old_and_new_config.conf')
+        # write some throttling values to the config file (not default ones)
         with open(conf_file, 'w') as fp:
             fp.write('[__main__]\n')
             fp.write('log_level = NOTE\n')
             fp.write('[logging]\n')
-            fp.write('level = DEBUG\n')
+            fp.write('level = ERROR\n')
         self.assertTrue(os.path.exists(conf_file))
         self.cp.read([conf_file])
         self.cp.parse_all()
-        self.assertEquals(self.cp.get('logging', 'level').value, 10)
+        self.assertEquals(self.cp.get('logging', 'level').value, logging.ERROR)
 
     def test_old_default_config(self):
         """Test log_level upgrade hook with an old default config"""
@@ -513,3 +592,14 @@ class SyncDaemonConfigParserTests(BaseTwistedTestCase):
         self.cp.parse_all()
         self.assertEquals(self.cp.get('__main__', 'ignore').value,
                           ['.*\\.pyc', '.*\\.sw[opnx]'])
+
+    def test_use_trash_default(self):
+        """Test default configuration for use_trash."""
+        self.cp.parse_all()
+        self.assertEqual(self.cp.get('__main__', 'use_trash').value, True)
+
+    def test_ignore_libreoffice_lockfiles(self):
+        """Test the default config includes ignoring libreoffice lockfiles."""
+        self.cp.parse_all()
+        self.assertIn(r'\A\.~lock\..*#\Z',
+                      self.cp.get('__main__', 'ignore').value)
