@@ -1,4 +1,8 @@
-# Copyright 2009-2010 Canonical Ltd.
+# ubuntuone.api.__init__ - load the correct api according to the platform
+#
+# Author: Manuel de la Pena <manuel@canonical.com>
+#
+# Copyright 2010 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -11,4 +15,4 @@
 #
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""Code to talk to public API in one.ubuntu.com"""
+
