@@ -62,6 +62,7 @@ CONFIG_LOGS = 'logging.conf'
 
 # sections
 THROTTLING = 'bandwidth_throttling'
+NOTIFICATIONS = 'notifications'
 MAIN = '__main__'
 
 # global logger
@@ -97,11 +98,16 @@ def xdg_data_dir_parser(value):
 
 
 def log_level_parser(value):
-    """perser for "logging" module log levels."""
-    level = getattr(logging, value, None)
-    if level is None:
+    """Parser for "logging" module log levels.
+
+    The logging API sucks big time, the only way to trustworthy find if the
+    log level is defined is to check the private attribute.
+    """
+    try:
+        level = logging._levelNames[value]
+    except KeyError:
         # if level don't exists in our custom levels, fallback to DEBUG
-        level = getattr(logging, 'DEBUG')
+        level = logging.DEBUG
     return level
 
 
@@ -141,13 +147,14 @@ def get_config_files():
     # reverse the list as load_config_paths returns the user dir first
     config_files.reverse()
     # if we are running from a branch, get the config files from it too
-    config_file = os.path.join('data', CONFIG_FILE)
+    config_file = os.path.join(os.path.dirname(__file__), os.path.pardir,
+                               os.path.pardir, 'data', CONFIG_FILE)
     if os.path.exists(config_file):
         config_files.append(config_file)
 
     config_logs = os.path.join('data', CONFIG_LOGS)
     if os.path.exists(config_logs):
-        config_files.append(config_file)
+        config_files.append(config_logs)
 
     return config_files
 
@@ -266,7 +273,7 @@ class _Config(SyncDaemonConfigParser):
     def save(self):
         """Save the config object to disk"""
         # cleanup empty sections
-        for section in [MAIN, THROTTLING]:
+        for section in [MAIN, THROTTLING, NOTIFICATIONS]:
             if self.has_section(section) and not self.options(section):
                 self.remove_section(section)
         with open(self.config_file+'.new', 'w') as fp:
@@ -334,6 +341,14 @@ class _Config(SyncDaemonConfigParser):
     def get_udf_autosubscribe(self):
         return self.get_parsed(MAIN, 'udf_autosubscribe')
 
+    @requires_section(MAIN)
+    def set_share_autosubscribe(self, enabled):
+        self.set(MAIN, 'share_autosubscribe', str(enabled))
+
+    @requires_section(MAIN)
+    def get_share_autosubscribe(self):
+        return self.get_parsed(MAIN, 'share_autosubscribe')
+
     # files sync enablement get/set
     @requires_section(MAIN)
     def set_files_sync_enabled(self, enabled):
@@ -343,6 +358,32 @@ class _Config(SyncDaemonConfigParser):
     def get_files_sync_enabled(self):
         return self.get_parsed(MAIN, 'files_sync_enabled')
 
+    @requires_section(MAIN)
+    def set_autoconnect(self, enabled):
+        self.set(MAIN, 'autoconnect', str(enabled))
+
+    @requires_section(MAIN)
+    def get_autoconnect(self):
+        return self.get_parsed(MAIN, 'autoconnect')
+
+    @requires_section(MAIN)
+    def get_use_trash(self):
+        return self.get_parsed(MAIN, 'use_trash')
+
+    @requires_section(MAIN)
+    def set_use_trash(self, enabled):
+        self.set(MAIN, 'use_trash', str(enabled))
+
+    @requires_section(NOTIFICATIONS)
+    def set_show_all_notifications(self, enabled):
+        self.set(NOTIFICATIONS, 'show_all_notifications', str(enabled))
+
+    @requires_section(NOTIFICATIONS)
+    def get_show_all_notifications(self):
+        """The value of the show_all_notifications setting."""
+        return self.get_parsed(NOTIFICATIONS, 'show_all_notifications')
+
+
 
 def configglue(fileobj, *filenames, **kwargs):
     """Populate an OptionParser with options and defaults taken from a
