@@ -29,7 +29,14 @@ from ubuntuone.syncdaemon.filesystem_manager import get_stat
 from twisted.internet import defer, reactor
 
 from ubuntuone.syncdaemon.interfaces import IMarker
-
+from ubuntuone.platform import (
+    access,
+    listdir,
+    path_exists,
+    remove_file,
+    rename,
+    stat_path,
+)
 
 class ScanTransactionDirty(Exception):
     """The transaction was dirty."""
@@ -40,6 +47,7 @@ class ScanNoDirectory(Exception):
 # local rescan logger
 lr_logger = logging.getLogger('ubuntuone.SyncDaemon.local_rescan')
 log_info = functools.partial(lr_logger.log, logging.INFO)
+log_trace = functools.partial(lr_logger.log, logging.getLevelName('TRACE'))
 log_debug = functools.partial(lr_logger.log, logging.DEBUG)
 log_error = functools.partial(lr_logger.log, logging.ERROR)
 log_warning = functools.partial(lr_logger.log, logging.WARNING)
@@ -64,7 +72,7 @@ class LocalRescan(object):
         to_scan = self._get_volumes(all_volumes=False)
         for vol in to_scan:
             # check that the path exists in disk
-            if not os.path.exists(vol.path):
+            if not path_exists(vol.path):
                 log_warning('Volume dissapeared: %r - %r', vol.volume_id, vol.path)
                 if isinstance(vol, volume_manager.Share):
                     log_debug('Removing %r metadata', vol.volume_id)
@@ -100,7 +108,7 @@ class LocalRescan(object):
         fix the nodes states, we just remove them to don't let garbage behind.
         """
         try:
-            partials = os.listdir(self.fsm.partials_dir)
+            partials = listdir(self.fsm.partials_dir)
         except OSError, e:
             if e.errno != errno.ENOENT:
                 raise
@@ -110,28 +118,30 @@ class LocalRescan(object):
         for partial in partials:
             partial_path = os.path.join(self.fsm.partials_dir, partial)
             log_debug("Removing old .partial: %r", partial_path)
-            os.remove(partial_path)
+            remove_file(partial_path)
 
     def _process_limbo(self):
         """Process the FSM limbos and send corresponding AQ orders."""
         log_info("processing trash")
-        trash_log = "share_id=%r  parent_id=%r  node_id=%r"
-        for share_id, node_id, parent_id in self.fsm.get_iter_trash():
-            datalog = trash_log % (share_id, parent_id, node_id)
+        trash_log = "share_id=%r  parent_id=%r  node_id=%r  path=%r"
+        for share_id, node_id, parent_id, path, is_dir in \
+                                                self.fsm.get_iter_trash():
+            datalog = trash_log % (share_id, parent_id, node_id, path)
             if IMarker.providedBy(node_id) or IMarker.providedBy(parent_id):
                 # situation where the node is not in the server
                 log_info("removing from trash: " + datalog)
                 self.fsm.remove_from_trash(share_id, node_id)
                 continue
             log_info("generating Unlink from trash: " + datalog)
-            self.aq.unlink(share_id, parent_id, node_id)
+            self.aq.unlink(share_id, parent_id, node_id, path, is_dir)
 
         log_info("processing move limbo")
         move_log = ("share_id=%r  node_id=%r  old_parent_id=%r  "
-                    "new_parent_id=%r  new_name=%r")
+                    "new_parent_id=%r  new_name=%r  path_from=%r  path_to=%r")
         for data in self.fsm.get_iter_move_limbo():
             to_log = move_log % data
-            share_id, node_id, old_parent_id, new_parent_id, new_name = data
+            (share_id, node_id, old_parent_id, new_parent_id, new_name,
+                                                    path_from, path_to) = data
             maybe_markers = (share_id, node_id, old_parent_id, new_parent_id)
             if any(IMarker.providedBy(x) for x in maybe_markers):
                 # situation where the move was not ready
@@ -140,7 +150,7 @@ class LocalRescan(object):
                 continue
             log_info("generating Move from limbo: " + to_log)
             self.aq.move(share_id, node_id, old_parent_id,
-                         new_parent_id, new_name)
+                    new_parent_id, new_name, path_from, path_to)
 
     def _process_ro_shares(self):
         """Process ro shares and reschedule interrupted downloads."""
@@ -216,7 +226,7 @@ class LocalRescan(object):
 
     def _send_scan_error(self, mdid, udfmode):
         """Sends the scan error event."""
-        self.eq.push("LR_SCAN_ERROR", mdid, udfmode)
+        self.eq.push("LR_SCAN_ERROR", mdid=mdid, udfmode=udfmode)
 
     def _queue_scan(self):
         """If there's a scan in progress, queue the new one for later."""
@@ -232,26 +242,26 @@ class LocalRescan(object):
         def revert_watches():
             """Restore the just added watches and unsubscribe volume."""
             for path in added_watches:
-                self.eq.inotify_rm_watch(path)
+                self.eq.rm_watch(path)
             m = "Unsubscribing UDF %r because not in disk: %r"
             log_info(m, volume.volume_id, volume.path)
             self.vm.unsubscribe_udf(volume.volume_id)
 
         for ancestor in volume.ancestors:
             # check that ancestor is still there
-            if not os.access(ancestor, os.R_OK):
+            if not access(ancestor):
                 log_info("Tree broken at path: %r", volume.path)
                 revert_watches()
                 return False
 
             # add watch if needed
-            log_debug("Adding watch to UDF's ancestor %r", ancestor)
-            if not self.eq.inotify_has_watch(ancestor):
+            if not self.eq.has_watch(ancestor):
+                log_debug("Adding watch to UDF's ancestor %r", ancestor)
                 added_watches.append(ancestor)
-                self.eq.inotify_add_watch(ancestor)
+                self.eq.add_watch(ancestor)
 
         # finally, check that UDF is ok in disk
-        if not os.access(volume.path, os.R_OK):
+        if not access(volume.path):
             revert_watches()
             return False
 
@@ -341,14 +351,16 @@ class LocalRescan(object):
         fh_factory = lambda: self.fsm.get_partial_for_writing(mdobj.node_id,
                                                               mdobj.share_id)
         self.aq.download(mdobj.share_id, mdobj.node_id,
-                         mdobj.server_hash, fh_factory)
+                         mdobj.server_hash, fullname, fh_factory)
 
     def _resume_upload(self, fullname):
         """Resume an interrupted upload."""
         mdobj = self.fsm.get_by_path(fullname)
         fh_factory = lambda: self.fsm.open_file(mdobj.mdid)
+        upload_id = getattr(mdobj, 'upload_id', None)
         self.aq.upload(mdobj.share_id, mdobj.node_id, mdobj.server_hash,
-                       mdobj.local_hash, mdobj.crc32, mdobj.size, fh_factory)
+                       mdobj.local_hash, mdobj.crc32, mdobj.size,
+                       fullname, fh_factory, upload_id=upload_id)
 
     def check_stat(self, fullname, oldstat):
         """Check stat info and return if different.
@@ -361,7 +373,7 @@ class LocalRescan(object):
         """
         if oldstat is None:
             return True
-        newstat = os.stat(fullname)
+        newstat = stat_path(fullname)
         different = (newstat.st_ino != oldstat.st_ino or
                      newstat.st_size != oldstat.st_size or
                      newstat.st_mtime != oldstat.st_mtime)
@@ -394,14 +406,14 @@ class LocalRescan(object):
             # remove fullname after removing its children,
             # otherwise metadata removal may fail
             log_info(message, fullname)
-            os.rename(fullname, fullname + ".u1conflict")
+            rename(fullname, fullname + ".u1conflict")
             self.fsm.delete_metadata(fullname)
 
             # if asked, remove also that file (if still exists)
             if also_remove is not None:
                 try:
                     log_info("Also remove %r", also_remove)
-                    os.remove(also_remove)
+                    remove_file(also_remove)
                 except OSError, e:
                     if e.errno != errno.ENOENT:
                         raise
@@ -447,7 +459,7 @@ class LocalRescan(object):
                     to_scan_later.append(fullname)
                 elif changed == "NONE":
                     # it's old, we should scan it later
-                    log_debug("comp yield: dir %r will be scaned later "
+                    log_trace("comp yield: dir %r will be scaned later "
                               "because it's in NONE!", fullname)
                     to_scan_later.append(fullname)
                 else:
@@ -472,15 +484,25 @@ class LocalRescan(object):
                     events.append(('FS_FILE_CREATE', fullname))
                     events.append(('FS_FILE_CLOSE_WRITE', fullname))
                 elif changed == "LOCAL":
-                    log_info("Resuming upload because it was interrupted: %r",
-                             fullname)
-                    self._resume_upload(fullname)
+                    different = self.check_stat(fullname, statinfo)
+                    if different:
+                        # hash it to see the changes, Sync will take care
+                        log_debug("comp yield: file %r in LOCAL and changed",
+                                  fullname)
+                        events.append(('FS_FILE_CLOSE_WRITE', fullname))
+                    else:
+                        # file didn't change, resume upload
+                        log_debug("resuming upload because it was "
+                                  "interrupted: %r", fullname)
+                        self._resume_upload(fullname)
                 elif changed == "NONE":
                     # what about stat info?
-                    log_debug("comp yield: file %r was here, let's check stat",
+                    log_trace("comp yield: file %r was here, let's check stat",
                               fullname)
                     different = self.check_stat(fullname, statinfo)
                     if different:
+                        log_debug("comp yield: file content changed: %r",
+                                  fullname)
                         events.append(('FS_FILE_CLOSE_WRITE', fullname))
                     # no 'else' here: the file is the same as before, all ok
                 elif changed == "SERVER":
@@ -597,11 +619,11 @@ class LocalRescan(object):
         """Gets one dir and compares with fsm."""
         share, dirpath, udf_mode = scan_info
 
-        if self.eq.inotify_has_watch(dirpath):
+        if self.eq.has_watch(dirpath):
             log_debug("Path already has watch: %r", dirpath)
         else:
             log_debug("Adding watch to %r", dirpath)
-            self.eq.inotify_add_watch(dirpath)
+            self.eq.add_watch(dirpath)
 
         to_later = []
         self.eq.freeze_begin(dirpath)
@@ -610,18 +632,18 @@ class LocalRescan(object):
             """The scan, really."""
 
             log_debug("scanning the dir %r", dirpath)
-            listdir = os.listdir(dirpath)
+            dircontent = listdir(dirpath)
 
             # get the info from disk
             dnames = []
             fnames = []
-            for something in listdir:
+            for something in dircontent:
                 fullname = os.path.join(dirpath, something)
                 stat_result = get_stat(fullname)
                 if stat_result is None or \
                    stat.S_ISLNK(stat_result.st_mode):
                     continue
-                elif not os.access(fullname, os.R_OK):
+                elif not access(fullname):
                     log_warning("Ignoring path as we don't have enough "
                                 "permissions to track it: %r", fullname)
                     continue
