@@ -226,14 +226,9 @@ class QueueManager(object):
     """A smaller finite state machine to handle queues."""
 
     IDLE = Node('IDLE', "nothing in the queues")
-    WORKING_ON_METADATA = Node('WORKING_ON_METADATA', "working on metadata")
-    WORKING_ON_CONTENT = Node('WORKING_ON_CONTENT', "working con content")
-    WORKING_ON_BOTH = Node('WORKING_ON_BOTH', "working on both")
+    WORKING = Node('WORKING', "working on the commands queue")
 
-    def __init__(self, state_manager):
-        self.sm = state_manager
-        self.cq = state_manager.aq.content_queue
-        self.mq = state_manager.aq.meta_queue
+    def __init__(self):
         self.state = self.IDLE
         self.log = logging.getLogger("ubuntuone.SyncDaemon.QueueManager")
         self.log.debug("start")
@@ -244,49 +239,17 @@ class QueueManager(object):
         self.state = new_state
 
     def on_event(self, event):
-        """Handle transitions between working queues."""
+        """Handle transitions."""
         prv_state = self.state
         if self.state == self.IDLE:
-            if event == 'SYS_META_QUEUE_WAITING':
-                self._set_state(self.WORKING_ON_METADATA)
-                self._run(self.mq)
-            elif event == 'SYS_CONTENT_QUEUE_WAITING':
-                self._set_state(self.WORKING_ON_CONTENT)
-                self._run(self.cq)
+            if event == 'SYS_QUEUE_WAITING':
+                self._set_state(self.WORKING)
             else:
                 self._bad_event(event)
 
-        elif self.state == self.WORKING_ON_METADATA:
-            if event == 'SYS_META_QUEUE_WAITING':
-                self._run(self.mq)
-            elif event == 'SYS_META_QUEUE_DONE':
+        elif self.state == self.WORKING:
+            if event == 'SYS_QUEUE_DONE':
                 self._set_state(self.IDLE)
-            elif event == 'SYS_CONTENT_QUEUE_WAITING':
-                self._set_state(self.WORKING_ON_BOTH)
-            else:
-                self._bad_event(event)
-
-        elif self.state == self.WORKING_ON_CONTENT:
-            if event == 'SYS_META_QUEUE_WAITING':
-                self._set_state(self.WORKING_ON_BOTH)
-                self._run(self.mq)
-            elif event == 'SYS_CONTENT_QUEUE_WAITING':
-                self._run(self.cq)
-            elif event == 'SYS_CONTENT_QUEUE_DONE':
-                self._set_state(self.IDLE)
-            else:
-                self._bad_event(event)
-
-        elif self.state == self.WORKING_ON_BOTH:
-            if event == 'SYS_META_QUEUE_WAITING':
-                self._run(self.mq)
-            elif event == 'SYS_META_QUEUE_DONE':
-                self._set_state(self.WORKING_ON_CONTENT)
-                self._run(self.cq)
-            elif event == 'SYS_CONTENT_QUEUE_WAITING':
-                pass # same node, no run()
-            elif event == 'SYS_CONTENT_QUEUE_DONE':
-                self._set_state(self.WORKING_ON_METADATA)
             else:
                 self._bad_event(event)
 
@@ -296,32 +259,11 @@ class QueueManager(object):
         # inform if changed
         return self.state != prv_state
 
-    def on_enter(self, new_node):
-        """Called when SM gets into a new node."""
-        if new_node == StateManager.QUEUE_MANAGER:
-            if self.state == self.WORKING_ON_CONTENT:
-                self._run(self.cq)
-            elif self.state in (self.WORKING_ON_METADATA, self.WORKING_ON_BOTH):
-                self._run(self.mq)
-
     def _bad_event(self, event):
         """Log the bad event."""
         m = "Bad Event received: Got %r while in %s"
         self.log.warning(m, event, self.state)
 
-    def _run(self, queue):
-        """Execute queue.run() if conditions are ok.
-
-        Conditions:
-
-        - the queue needs to have something to run
-
-        - the global state should be this QueueManager
-        """
-        if len(queue) and self.sm.state == StateManager.QUEUE_MANAGER:
-            self.log.debug("In %s: running %s", self.state.name, queue.name)
-            queue.run()
-
 
 ACCEPTED_EVENTS = [
     'SYS_AUTH_ERROR',
@@ -330,17 +272,16 @@ ACCEPTED_EVENTS = [
     'SYS_CONNECTION_LOST',
     'SYS_CONNECTION_MADE',
     'SYS_CONNECTION_RETRY',
-    'SYS_CONTENT_QUEUE_DONE',
-    'SYS_CONTENT_QUEUE_WAITING',
+    'SYS_QUEUE_DONE',
+    'SYS_QUEUE_WAITING',
     'SYS_HANDSHAKE_TIMEOUT',
     'SYS_INIT_DONE',
     'SYS_LOCAL_RESCAN_DONE',
-    'SYS_META_QUEUE_DONE',
-    'SYS_META_QUEUE_WAITING',
     'SYS_NET_CONNECTED',
     'SYS_NET_DISCONNECTED',
     'SYS_PROTOCOL_VERSION_ERROR',
     'SYS_PROTOCOL_VERSION_OK',
+    'SYS_QUIT',
     'SYS_ROOT_MISMATCH',
     'SYS_SERVER_RESCAN_DONE',
     'SYS_SERVER_ERROR',
@@ -373,7 +314,7 @@ class StateManager(object):
 
     SERVER_RESCAN = Node('SERVER_RESCAN', "doing server rescan", conn=True)
 
-    QUEUE_MANAGER = Node('QUEUE_MANAGER', "processing queues",
+    QUEUE_MANAGER = Node('QUEUE_MANAGER', "processing the commands pool",
                          conn=True, online=True)
 
     STANDOFF = Node('STANDOFF', "waiting for connection to end", conn=True)
@@ -382,14 +323,16 @@ class StateManager(object):
                          "different", error=True)
     UNKNOWN_ERROR = Node('UNKNOWN_ERROR', "something went wrong", error=True)
 
+    SHUTDOWN = Node('SHUTDOWN', "shutting down the service")
+
     def __init__(self, main, handshake_timeout=None):
         self.main = main
         self.aq = main.action_q
         self.eq = main.event_q
-        self.eq.subscribe(self)
-        self.connection = ConnectionManager(self, handshake_timeout)
-        self.queues = QueueManager(self)
         self.state = self.INIT
+        self.queues = QueueManager()
+        self.connection = ConnectionManager(self, handshake_timeout)
+        self.eq.subscribe(self)
         self.log = logging.getLogger("ubuntuone.SyncDaemon.StateManager")
         self.log.debug("start")
 
@@ -428,12 +371,17 @@ class StateManager(object):
             (self.SERVER_RESCAN, 'SYS_SERVER_ERROR'): self.STANDOFF,
         }
 
-    def handle_default(self, event, *args, **kwargs):
+    def handle_default(self, event, **kwargs):
         """Receive all the events to make States tick."""
         if event not in ACCEPTED_EVENTS:
             return
         self.log.debug("received event %r", event)
 
+        # quit
+        if event == 'SYS_QUIT':
+            self._transition(event, StateManager.SHUTDOWN)
+            return
+
         # error management
         if event == 'SYS_UNKNOWN_ERROR':
             self._transition(event, StateManager.UNKNOWN_ERROR)
@@ -445,8 +393,7 @@ class StateManager(object):
             return
 
         # queue events
-        if event in ('SYS_META_QUEUE_WAITING', 'SYS_META_QUEUE_DONE',
-                     'SYS_CONTENT_QUEUE_WAITING', 'SYS_CONTENT_QUEUE_DONE'):
+        if event in ('SYS_QUEUE_WAITING', 'SYS_QUEUE_DONE'):
             self.log.debug("sending event to QueueManager")
             changed = self.queues.on_event(event)
             if changed:
@@ -502,7 +449,7 @@ class StateManager(object):
 
         # on exit actions
         if self.state == self.QUEUE_MANAGER:
-            self.aq.cleanup()
+            self.aq.queue.stop()
 
         # make the transition
         self.state = new_node
@@ -521,11 +468,12 @@ class StateManager(object):
             self.main.authenticate()
         elif new_node == self.SERVER_RESCAN:
             self.main.server_rescan()
+        elif new_node == self.QUEUE_MANAGER:
+            self.aq.queue.run()
         elif new_node == self.STANDOFF:
             self.main.action_q.disconnect()
         elif new_node == self.UNKNOWN_ERROR:
             self.main.restart()
-        self.queues.on_enter(new_node)
         self.connection.on_enter(new_node)
 
         # inform the system
