@@ -30,8 +30,14 @@ from ubuntuone.syncdaemon.fsm.fsm import \
     StateMachineRunner, StateMachine
 from ubuntuone.syncdaemon import u1fsfsm
 from ubuntuone.syncdaemon.logger import DebugCapture
-from ubuntuone.syncdaemon.filesystem_manager import InconsistencyError
+from ubuntuone.syncdaemon.filesystem_manager import (
+    DirectoryNotRemovable,
+    InconsistencyError,
+)
 from ubuntuone.syncdaemon.volume_manager import VolumeDoesNotExist
+from ubuntuone.platform import (
+    stat_path,
+)
 
 empty_hash = ""
 
@@ -307,7 +313,7 @@ class SyncStateMachineRunner(StateMachineRunner):
 
     def validate_actual_data(self, path, oldstat):
         """Validates that the received info is not obsolete."""
-        newstat = os.stat(path)
+        newstat = stat_path(path)
         if newstat.st_ino != oldstat.st_ino or \
            newstat.st_size != oldstat.st_size or \
            newstat.st_mtime != oldstat.st_mtime:
@@ -450,15 +456,15 @@ class SyncStateMachineRunner(StateMachineRunner):
         """Get the contents for the file."""
         self.key.set(server_hash=hash)
         self.key.sync()
-        self.m.fs.create_partial(node_id=self.key['node_id'],
-                                 share_id=self.key['share_id'])
-        self.m.action_q.download(
-            share_id=self.key['share_id'], node_id=self.key['node_id'],
-            server_hash=hash,
-            fileobj_factory=lambda: self.m.fs.get_partial_for_writing(
-                node_id=self.key['node_id'],
-                share_id=self.key['share_id'])
-            )
+        share_id = self.key['share_id']
+        node_id = self.key['node_id']
+        path = self.key['path']
+        self.m.fs.create_partial(node_id=node_id, share_id=share_id)
+        factory = lambda: self.m.fs.get_partial_for_writing(node_id=node_id,
+                                                            share_id=share_id)
+        self.m.action_q.download(share_id=share_id, node_id=node_id,
+                                 server_hash=hash, path=path,
+                                 fileobj_factory=factory)
 
     def reget_file(self, event, params, hash):
         """cancel and reget this download."""
@@ -482,9 +488,10 @@ class SyncStateMachineRunner(StateMachineRunner):
         node_id = self.key['node_id']
 
         self.m.action_q.move(share_id, node_id, old_parent_id,
-                             new_parent_id, new_name)
+                             new_parent_id, new_name, path_from, path_to)
         self.m.fs.add_to_move_limbo(share_id, node_id, old_parent_id,
-                                    new_parent_id, new_name)
+                                    new_parent_id, new_name,
+                                    path_from, path_to)
         self.key.moved(share_id, path_to)
 
         # we only hash it if we're a file, not a directory
@@ -524,7 +531,7 @@ class SyncStateMachineRunner(StateMachineRunner):
         self.key.sync()
         name = os.path.basename(path)
         marker = MDMarker(self.key.get_mdid())
-        self.m.action_q.make_file(share_id, parent_id, name, marker)
+        self.m.action_q.make_file(share_id, parent_id, name, marker, path)
 
     def release_marker_ok(self, event, parms, new_id, marker):
         """Release ok the received marker in AQ's DeferredMap."""
@@ -541,6 +548,11 @@ class SyncStateMachineRunner(StateMachineRunner):
         self.m.action_q.uuid_map.set(marker, new_id)
         self.m.fs.set_node_id(self.key['path'], new_id)
 
+    def new_server_file_having_local(self, event, parms, share_id, node_id,
+                                     parent_id, name):
+        """Got new file from server, we have it local but without id yet."""
+        self.m.fs.set_node_id(self.key['path'], node_id)
+
     def new_local_dir(self, event, parms, path):
         """a new local dir was created"""
         parent_path = os.path.dirname(path)
@@ -551,7 +563,7 @@ class SyncStateMachineRunner(StateMachineRunner):
         name = os.path.basename(path)
         mdid = self.key.get_mdid()
         marker = MDMarker(mdid)
-        self.m.action_q.make_dir(share_id, parent_id, name, marker)
+        self.m.action_q.make_dir(share_id, parent_id, name, marker, path)
         self.m.lr.scan_dir(mdid, path)
 
     def new_local_dir_created(self, event, parms, new_id, marker):
@@ -569,26 +581,35 @@ class SyncStateMachineRunner(StateMachineRunner):
 
     def reput_file_from_local(self, event, params, hash):
         """Re put the file from its local state."""
+        self.m.action_q.cancel_upload(share_id=self.key['share_id'],
+                            node_id=self.key['node_id'])
+
         local_hash = self.key['local_hash']
         previous_hash = self.key['server_hash']
         crc32 = self.key['crc32']
         size = self.key['size']
         share_id = self.key['share_id']
         node_id = self.key['node_id']
+        path = self.key['path']
+        upload_id = self.key.get('upload_id')
 
         self.m.action_q.upload(share_id, node_id, previous_hash, local_hash,
-                               crc32, size, self.key.open_file)
+                               crc32, size, path, self.key.open_file,
+                               upload_id=upload_id)
 
     def put_file(self, event, params, hash, crc32, size, stat):
         """Upload the file to the server."""
         previous_hash = self.key['server_hash']
+        path = self.key['path']
+        upload_id = self.key.get('upload_id')
         self.key.set(local_hash=hash, stat=stat, crc32=crc32, size=size)
         self.key.sync()
 
         self.m.action_q.upload(share_id=self.key['share_id'],
             node_id=self.key['node_id'], previous_hash=previous_hash,
-            hash=hash, crc32=crc32, size=size,
-            fileobj_factory=self.key.open_file)
+            hash=hash, crc32=crc32, size=size, path=path,
+            fileobj_factory=self.key.open_file, upload_id=upload_id)
+
 
     def converges_to_server(self, event, params, hash, crc32, size, stat):
         """the local changes now match the server"""
@@ -614,12 +635,14 @@ class SyncStateMachineRunner(StateMachineRunner):
                             node_id=self.key['node_id'])
         previous_hash = self.key['server_hash']
 
+        path = self.key['path']
+        upload_id = self.key.get('upload_id')
         self.key.set(local_hash=hash, stat=stat, crc32=crc32, size=size)
         self.key.sync()
         self.m.action_q.upload(share_id=self.key['share_id'],
             node_id=self.key['node_id'], previous_hash=previous_hash,
-            hash=hash, crc32=crc32, size=size,
-            fileobj_factory=self.key.open_file)
+            hash=hash, crc32=crc32, size=size, path=path,
+            fileobj_factory=self.key.open_file, upload_id=upload_id)
 
     def server_file_now_matches(self, event, params, hash):
         """We got a server hash that matches local hash"""
@@ -643,12 +666,12 @@ class SyncStateMachineRunner(StateMachineRunner):
         """server file was deleted."""
         try:
             self.key.delete_file()
+        except DirectoryNotRemovable:
+            # directory not empty and with stuff that should not be deleted
+            self.key.move_to_conflict()
+            self.key.delete_metadata()
         except OSError, e:
-            if e.errno == 39:
-                # if directory not empty
-                self.key.move_to_conflict()
-                self.key.delete_metadata()
-            elif e.errno == 2:
+            if e.errno == 2:
                 # file gone
                 pass
             else:
@@ -674,38 +697,42 @@ class SyncStateMachineRunner(StateMachineRunner):
 
     def delete_on_server(self, event, params, path):
         """local file was deleted."""
+        is_dir = self.key.is_dir()
         self.m.action_q.unlink(self.key['share_id'],
                                self.key['parent_id'],
-                               self.key['node_id'])
+                               self.key['node_id'], path, is_dir)
         self.key.delete_to_trash()
 
     def deleted_dir_while_downloading(self, event, params, path):
         """kill it"""
+        is_dir = self.key.is_dir()
         self.m.action_q.cancel_download(share_id=self.key['share_id'],
                                         node_id=self.key['node_id'])
         self.key.remove_partial()
         self.m.action_q.unlink(self.key['share_id'],
                                self.key['parent_id'],
-                               self.key['node_id'])
+                               self.key['node_id'], path, is_dir)
         self.key.delete_to_trash()
 
     def cancel_download_and_delete_on_server(self, event, params, path):
         """cancel_download_and_delete_on_server"""
+        is_dir = self.key.is_dir()
         self.m.action_q.cancel_download(share_id=self.key['share_id'],
                                         node_id=self.key['node_id'])
         self.key.remove_partial()
         self.m.action_q.unlink(self.key['share_id'],
                                self.key['parent_id'],
-                               self.key['node_id'])
+                               self.key['node_id'], path, is_dir)
         self.key.delete_to_trash()
 
     def cancel_upload_and_delete_on_server(self, event, params, path):
         """cancel_download_and_delete_on_server"""
+        is_dir = self.key.is_dir()
         self.m.action_q.cancel_upload(share_id=self.key['share_id'],
                                       node_id=self.key['node_id'])
         self.m.action_q.unlink(self.key['share_id'],
                                self.key['parent_id'],
-                               self.key['node_id'])
+                               self.key['node_id'], path, is_dir)
         self.key.delete_to_trash()
 
     def remove_trash(self, event, params, share_id, node_id):
@@ -819,7 +846,31 @@ class Sync(object):
         key = FSKey(self.m.fs, path=path)
         log = FileLogger(self.logger, key)
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
+
+        # check if the node found by path has other node_id (which will
+        # cause a conflict); note that we don't get the node_id from the
+        # 'key' as it lies to us with the marker
+        try:
+            mdid = key.get_mdid()
+        except KeyError:
+            pass  # no md at all, didn't find any node by the path
+        else:
+            mdobj = self.m.fs.get_by_mdid(mdid)
+            if mdobj.node_id is not None:
+                if mdobj.node_id == node_id:
+                    raise ValueError("Found same node_id in handle_SV_FILE_NEW"
+                                     " (node_id=%s path=%r)" % (node_id, path))
+                # have metadata with *other* node_id
+                log.debug("Wanted to apply SV_FILE_NEW with node_id %s to node"
+                          "with path %r, but found it with other id: %s",
+                          node_id, path, mdobj.node_id)
+                key.delete_file()
+                return None
+
         ssmr.on_event("SV_FILE_NEW", {}, share_id, node_id, parent_id, name)
+        self.m.event_q.push('SV_FILE_NEW', volume_id=share_id,
+                            node_id=node_id, parent_id=parent_id, name=name)
+        return self.m.fs.get_by_node_id(share_id, node_id)
 
     def _handle_SV_DIR_NEW(self, share_id, node_id, parent_id, name):
         """on SV_DIR_NEW"""
@@ -829,13 +880,23 @@ class Sync(object):
         log = FileLogger(self.logger, key)
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.on_event("SV_DIR_NEW", {}, share_id, node_id, parent_id, name)
+        self.m.event_q.push('SV_DIR_NEW', volume_id=share_id,
+                            node_id=node_id, parent_id=parent_id, name=name)
+        return self.m.fs.get_by_node_id(share_id, node_id)
 
-    def _handle_SV_FILE_DELETED(self, share_id, node_id):
+    def _handle_SV_FILE_DELETED(self, share_id, node_id, is_dir):
         """on SV_FILE_DELETED. Not called by EQ anymore."""
         key = FSKey(self.m.fs, share_id=share_id, node_id=node_id)
+        try:
+            path = key["path"]
+        except KeyError:
+            path = ""
         log = FileLogger(self.logger, key)
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.on_event("SV_FILE_DELETED", {})
+        self.m.event_q.push('SV_FILE_DELETED', volume_id=share_id,
+                            node_id=node_id, was_dir=is_dir,
+                            old_path=path)
 
     def handle_AQ_DOWNLOAD_FINISHED(self, share_id, node_id, server_hash):
         """on AQ_DOWNLOAD_FINISHED"""
@@ -852,13 +913,6 @@ class Sync(object):
         ssmr.signal_event_with_error_and_hash("AQ_DOWNLOAD_ERROR", error,
                                               server_hash)
 
-    def handle_AQ_DOWNLOAD_CANCELLED(self, share_id, node_id, server_hash):
-        """on AQ_DOWNLOAD_CANCELLED"""
-        key = FSKey(self.m.fs, share_id=share_id, node_id=node_id)
-        log = FileLogger(self.logger, key)
-        ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
-        ssmr.signal_event_with_hash("AQ_DOWNLOAD_CANCELLED", server_hash)
-
     def handle_AQ_DOWNLOAD_DOES_NOT_EXIST(self, share_id, node_id):
         """on AQ_DOWNLOAD_DOES_NOT_EXIST."""
         key = FSKey(self.m.fs, share_id=share_id, node_id=node_id)
@@ -1002,7 +1056,7 @@ class Sync(object):
                       new_parent_id, new_name)
 
     def handle_AQ_UNLINK_OK(self, share_id, parent_id, node_id,
-                            new_generation):
+                            new_generation, was_dir, old_path):
         """On AQ_UNLINK_OK."""
         key = FSKey(self.m.fs, share_id=share_id, node_id=node_id)
         log = FileLogger(self.logger, key)
@@ -1068,7 +1122,8 @@ class Sync(object):
                 # about it
                 if not dt.is_live:
                     to_delete.append(dt)
-                    self._handle_SV_FILE_DELETED(dt.share_id, dt.node_id)
+                    self._handle_SV_FILE_DELETED(dt.share_id, dt.node_id,
+                                                 is_dir)
                     # nothing else should happen with this
                     continue
 
@@ -1082,13 +1137,14 @@ class Sync(object):
                         continue
 
                     # node not there, we must create it
+                    args = (dt.share_id, dt.node_id, dt.parent_id, dt_name)
                     if is_dir:
-                        self._handle_SV_DIR_NEW(dt.share_id, dt.node_id,
-                                                dt.parent_id, dt_name)
+                        node = self._handle_SV_DIR_NEW(*args)
                     else:
-                        self._handle_SV_FILE_NEW(dt.share_id, dt.node_id,
-                                                 dt.parent_id, dt_name)
-                    node = self.m.fs.get_by_node_id(dt.share_id, dt.node_id)
+                        node = self._handle_SV_FILE_NEW(*args)
+                    if node is None:
+                        # the node was not created!
+                        continue
 
                 # if the delta is older than the node, skip!
                 if node.generation > dt.generation:
@@ -1184,7 +1240,7 @@ class Sync(object):
                 continue
 
             if not node_id in live_nodes:
-                self._handle_SV_FILE_DELETED(volume_id, node_id)
+                self._handle_SV_FILE_DELETED(volume_id, node_id, node.is_dir)
                 deletes += 1
 
         self.logger.info(
